==> basex_8.2.3.orig.tar.gz: basex-8.2.3/basex-core/src/main/java/org/basex/query/util/fingertree/SingletonTree.java <==
package org.basex.query.util.fingertree;

/**
 * A tree consisting of a single value.
 *
 * @author BaseX Team 2005-15, BSD License
 * @author Leo Woerteler
 *
 * @param <N> node type
 * @param <E> element type
 */
final class SingletonTree<N, E> extends FingerTree<N, E> {
  /** The element. */
  final Node<N, E> elem;

  /**
   * Constructor.
   * @param elem element
   */
  public SingletonTree(final Node<N, E> elem) {
    this.elem = elem;
  }

  @Override
  public DeepTree<N, E> cons(final Node<N, E> fst) {
    final long leftSize = fst.size();
    @SuppressWarnings("unchecked")
    final Node<N, E>[] left = new Node[] { fst }, right = new Node[] { elem };
    return DeepTree.get(left, leftSize, right, leftSize + elem.size());
  }

  @Override
  public DeepTree<N, E> snoc(final Node<N, E> lst) {
    final long leftSize = elem.size();
    @SuppressWarnings("unchecked")
    final Node<N, E>[] left = new Node[] { elem }, right = new Node[] { lst };
    return DeepTree.get(left, leftSize, right, leftSize + lst.size());
  }

  @Override
  public Node<N, E> head() {
    return elem;
  }

  @Override
  public Node<N, E> last() {
    return elem;
  }

  @Override
  public FingerTree<N, E> init() {
    return EmptyTree.getInstance();
  }

  @Override
  public FingerTree<N, E> tail() {
    return EmptyTree.getInstance();
  }

  @Override
  public long size() {
    return elem.size();
  }

  @Override
  public FingerTree<N, E> concat(final Node<N, E>[] mid, final long sz,
      final FingerTree<N, E> other) {
    return other.isEmpty() ? addAll(mid, sz, false) : other.addAll(mid, sz, true).cons(elem);
  }

  @Override
  public FingerTree<N, E> reverse() {
    return new SingletonTree<>(elem.reverse());
  }

  @Override
  public FingerTree<N, E> insert(final long pos, final E val) {
    @SuppressWarnings("unchecked")
    final Node<N, E>[] siblings = new Node[4];
    if(!elem.insert(siblings, pos, val)) {
      // node was not split
      return new SingletonTree<>(siblings[1]);
    }

    final Node<N, E> l = siblings[1], r = siblings[2];
    @SuppressWarnings("unchecked")
    final Node<N, E>[] left = new Node[] { l }, right = new Node[] { r };
    return DeepTree.get(left, l.size(), right, elem.size() + 1);
  }

  @Override
  public TreeSlice<N, E> remove(final long pos) {
    final NodeLike<N, E>[] removed = elem.remove(null, null, pos);
    return new TreeSlice<>(removed[1]);
  }

  @Override
  public TreeSlice<N, E> slice(final long pos, final long len) {
    if(pos == 0 && len == elem.size()) return new TreeSlice<>(this);
    return new TreeSlice<>(elem.slice(pos, len));
  }

  @Override
  FingerTree<N, E> addAll(final Node<N, E>[] nodes, final long sz, final boolean left) {
    if(nodes.length == 0) return this;
    if(nodes.length <= MAX_DIGIT) {
      @SuppressWarnings("unchecked")
      final Node<N, E>[] arr = new Node[] { elem };
      return left ? DeepTree.get(nodes, arr) : DeepTree.get(arr, nodes);
    }

    final FingerTree<N, E> tree = buildTree(nodes, nodes.length, sz);
    return left ? tree.snoc(elem) : tree.cons(elem);
  }

  @Override
  public FingerTree<N, E> replaceHead(final Node<N, E> head) {
    return new SingletonTree<>(head);
  }

  @Override
  public FingerTree<N, E> replaceLast(final Node<N, E> head) {
    return new SingletonTree<>(head);
  }

  @Override
  void toString(final StringBuilder sb, final int indent) {
    for(int i = 0; i < indent; i++) sb.append("  ");
    sb.append("Single[\n");
    toString(elem, sb, indent + 1);
    sb.append("\n");
    for(int i = 0; i < indent; i++) sb.append("  ");
    sb.append("]");
  }

  @Override
  public long checkInvariants() {
    return elem.checkInvariants();
  }
}

==> basex_8.5.1.orig.tar.gz: basex-8.5.1/basex-core/src/main/java/org/basex/query/util/fingertree/SingletonTree.java <==
package org.basex.query.util.fingertree;

/**
 * A tree consisting of a single value.
 *
 * @author BaseX Team 2005-16, BSD License
 * @author Leo Woerteler
 *
 * @param <N> node type
 * @param <E> element type
 */
final class SingletonTree<N, E> extends FingerTree<N, E> {
  /** The element. */
  final Node<N, E> elem;

  /**
   * Constructor.
   * @param elem element
   */
  SingletonTree(final Node<N, E> elem) {
    this.elem = elem;
  }

  @Override
  public DeepTree<N, E> cons(final Node<N, E> fst) {
    final long leftSize = fst.size();
    @SuppressWarnings("unchecked")
    final Node<N, E>[] left = new Node[] { fst }, right = new Node[] { elem };
    return DeepTree.get(left, leftSize, right, leftSize + elem.size());
  }

  @Override
  public DeepTree<N, E> snoc(final Node<N, E> lst) {
    final long leftSize = elem.size();
    @SuppressWarnings("unchecked")
    final Node<N, E>[] left = new Node[] { elem }, right = new Node[] { lst };
    return DeepTree.get(left, leftSize, right, leftSize + lst.size());
  }

  @Override
  public Node<N, E> head() {
    return elem;
  }

  @Override
  public Node<N, E> last() {
    return elem;
  }

  @Override
  public FingerTree<N, E> init() {
    return EmptyTree.getInstance();
  }

  @Override
  public FingerTree<N, E> tail() {
    return EmptyTree.getInstance();
  }

  @Override
  public long size() {
    return elem.size();
  }

  @Override
  public FingerTree<N, E> concat(final Node<N, E>[] mid, final long sz,
      final FingerTree<N, E> other) {
    return other.isEmpty() ? addAll(mid, sz, false) : other.addAll(mid, sz, true).cons(elem);
  }

  @Override
  public FingerTree<N, E> reverse() {
    return new SingletonTree<>(elem.reverse());
  }

  @Override
  public FingerTree<N, E> set(final long pos, final E val) {
    final long size = elem.size();
    if(pos < 0 || pos >= size) throw new IndexOutOfBoundsException(pos + ", size = " + size);
    return new SingletonTree<>(elem.set(pos, val));
  }

  @Override
  public FingerTree<N, E> insert(final long pos, final E val) {
    @SuppressWarnings("unchecked")
    final Node<N, E>[] siblings = new Node[4];
    if(!elem.insert(siblings, pos, val)) {
      // node was not split
      return new SingletonTree<>(siblings[1]);
    }

    final Node<N, E> l = siblings[1], r = siblings[2];
    @SuppressWarnings("unchecked")
    final Node<N, E>[] left = new Node[] { l }, right = new Node[] { r };
    return DeepTree.get(left, l.size(), right, elem.size() + 1);
  }

  @Override
  public TreeSlice<N, E> remove(final long pos) {
    final NodeLike<N, E>[] removed = elem.remove(null, null, pos);
    return new TreeSlice<>(removed[1]);
  }

  @Override
  public TreeSlice<N, E> slice(final long pos, final long len) {
    if(pos == 0 && len == elem.size()) return new TreeSlice<>(this);
    return new TreeSlice<>(elem.slice(pos, len));
  }

  @Override
  FingerTree<N, E> addAll(final Node<N, E>[] nodes, final long sz, final boolean left) {
    if(nodes.length == 0) return this;
    if(nodes.length <= MAX_DIGIT) {
      @SuppressWarnings("unchecked")
      final Node<N, E>[] arr = new Node[] { elem };
      return left ? DeepTree.get(nodes, arr) : DeepTree.get(arr, nodes);
    }

    final FingerTree<N, E> tree = buildTree(nodes, nodes.length, sz);
    return left ? tree.snoc(elem) : tree.cons(elem);
  }

  @Override
  public FingerTree<N, E> replaceHead(final Node<N, E> head) {
    return new SingletonTree<>(head);
  }

  @Override
  public FingerTree<N, E> replaceLast(final Node<N, E> head) {
    return new SingletonTree<>(head);
  }

  @Override
  void toString(final StringBuilder sb, final int indent) {
    for(int i = 0; i < indent; i++) sb.append("  ");
    sb.append("Single[\n");
    toString(elem, sb, indent + 1);
    sb.append("\n");
    for(int i = 0; i < indent; i++) sb.append("  ");
    sb.append("]");
  }

  @Override
  public long checkInvariants() {
    return elem.checkInvariants();
  }
}
